@@ -3,7 +3,7 @@
  * This file is part of gedit
  *
  * Copyright (C) 2002 Paolo Maggi and James Willcox
- * Copyright (C) 2003-2005 Paolo Maggi
+ * Copyright (C) 2003-2006 Paolo Maggi
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -22,11 +22,11 @@
  */
 
 /*
- * Modified by the gedit Team, 1998-2005. See the AUTHORS file for a 
+ * Modified by the gedit Team, 1998-2006. See the AUTHORS file for a 
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-plugin-manager.c,v 1.2 2005/12/12 17:27:55 pborelli Exp $
+ * $Id: gedit-plugin-manager.c 5402 2007-01-02 16:05:57Z paolo $
  */
 
 #ifdef HAVE_CONFIG_H
@@ -48,7 +48,8 @@
 enum
 {
 	ACTIVE_COLUMN,
-	NAME_COLUMN,
+	AVAILABLE_COLUMN,
+	INFO_COLUMN,
 	N_COLUMNS
 };
 
@@ -67,19 +68,23 @@ struct _GeditPluginManagerPrivate
 	const GList	*plugins;
 
 	GtkWidget 	*about;
+	
+	GtkWidget	*popup_menu;
 };
 
 G_DEFINE_TYPE(GeditPluginManager, gedit_plugin_manager, GTK_TYPE_VBOX)
 
 static GeditPluginInfo *plugin_manager_get_selected_plugin (GeditPluginManager *pm); 
 static void plugin_manager_toggle_active (GtkTreeIter *iter, GtkTreeModel *model); 
-static void plugin_manager_toggle_all (GeditPluginManager *pm); 
+static void gedit_plugin_manager_finalize (GObject *object);
 
 static void 
 gedit_plugin_manager_class_init (GeditPluginManagerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+	object_class->finalize = gedit_plugin_manager_finalize;
+
 	g_type_class_add_private (object_class, sizeof (GeditPluginManagerPrivate));
 }
 
@@ -88,7 +93,6 @@ about_button_cb (GtkWidget          *button,
 		 GeditPluginManager *pm)
 {
 	GeditPluginInfo *info;
-	GdkPixbuf *pixbuf = NULL;
 
 	gedit_debug (DEBUG_PLUGINS);
 
@@ -96,8 +100,6 @@ about_button_cb (GtkWidget          *button,
 
 	g_return_if_fail (info != NULL);
 
-	pixbuf = gdk_pixbuf_new_from_file (GNOME_ICONDIR "/gedit-plugin-manager.png", NULL);
-
 	/* if there is another about dialog already open destroy it */
 	if (pm->priv->about)
 		gtk_widget_destroy (pm->priv->about);
@@ -108,7 +110,7 @@ about_button_cb (GtkWidget          *button,
 		"authors", gedit_plugins_engine_get_plugin_authors (info),
 		"comments", gedit_plugins_engine_get_plugin_description (info),
 		"website", gedit_plugins_engine_get_plugin_website (info),
-		"logo", pixbuf,
+		"logo-icon-name", gedit_plugins_engine_get_plugin_icon_name (info),
 		NULL);
 
 	gtk_window_set_destroy_with_parent (GTK_WINDOW (pm->priv->about),
@@ -126,9 +128,6 @@ about_button_cb (GtkWidget          *button,
 	gtk_window_set_transient_for (GTK_WINDOW (pm->priv->about),
 				      GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET(pm))));
 	gtk_widget_show (pm->priv->about);
-
-	if (pixbuf != NULL)
-		g_object_unref (pixbuf);
 }
 
 static void
@@ -155,33 +154,62 @@ configure_button_cb (GtkWidget          *button,
 }
 
 static void
-plugin_manager_view_cell_cb (GtkTreeViewColumn *tree_column,
-			     GtkCellRenderer   *cell,
-			     GtkTreeModel      *tree_model,
-			     GtkTreeIter       *iter,
-			     gpointer           data)
+plugin_manager_view_info_cell_cb (GtkTreeViewColumn *tree_column,
+				  GtkCellRenderer   *cell,
+				  GtkTreeModel      *tree_model,
+				  GtkTreeIter       *iter,
+				  gpointer           data)
 {
 	GeditPluginInfo *info;
-//	const gchar *title;
+	gchar *text;
 	
 	g_return_if_fail (tree_model != NULL);
 	g_return_if_fail (tree_column != NULL);
 
-	gtk_tree_model_get (tree_model, iter, NAME_COLUMN, &info, -1);
+	gtk_tree_model_get (tree_model, iter, INFO_COLUMN, &info, -1);
 
 	if (info == NULL)
 		return;
 
-//	title = gtk_tree_view_column_get_title (tree_column);
+	text = g_markup_printf_escaped ("<b>%s</b>\n%s",
+					gedit_plugins_engine_get_plugin_name (info),
+					gedit_plugins_engine_get_plugin_description (info));
+	g_object_set (G_OBJECT (cell),
+		      "markup",
+		      text,
+		      "sensitive",
+		      gedit_plugins_engine_plugin_is_available (info),
+		      NULL);
 
-	/* FIXME: this string comparison stuff sucks.  is there a better way? */
-//	if (!strcmp (title, PLUGIN_MANAGER_NAME_TITLE))
-	g_object_set (G_OBJECT (cell), 
-		      "text", 
-		      gedit_plugins_engine_get_plugin_name (info), 
+	g_free (text);
+}
+
+static void
+plugin_manager_view_icon_cell_cb (GtkTreeViewColumn *tree_column,
+				  GtkCellRenderer   *cell,
+				  GtkTreeModel      *tree_model,
+				  GtkTreeIter       *iter,
+				  gpointer           data)
+{
+	GeditPluginInfo *info;
+	
+	g_return_if_fail (tree_model != NULL);
+	g_return_if_fail (tree_column != NULL);
+
+	gtk_tree_model_get (tree_model, iter, INFO_COLUMN, &info, -1);
+
+	if (info == NULL)
+		return;
+
+	g_object_set (G_OBJECT (cell),
+		      "icon-name",
+		      gedit_plugins_engine_get_plugin_icon_name (info),
+		      "sensitive",
+		      gedit_plugins_engine_plugin_is_available (info),
 		      NULL);
 }
 
+
 static void
 active_toggled_cb (GtkCellRendererToggle *cell,
 		   gchar                 *path_str,
@@ -247,19 +275,6 @@ row_activated_cb (GtkTreeView       *tree_view,
 	plugin_manager_toggle_active (&iter, model);
 }
 
-static void
-column_clicked_cb (GtkTreeViewColumn *tree_column,
-		   gpointer           data)
-{
-	GeditPluginManager *pm = data;
-
-	gedit_debug (DEBUG_PLUGINS);
-
-	g_return_if_fail (pm != NULL);
-
-	plugin_manager_toggle_all (pm);
-}
-
 static void
 plugin_manager_populate_lists (GeditPluginManager *pm)
 {
@@ -281,7 +296,8 @@ plugin_manager_populate_lists (GeditPluginManager *pm)
 		gtk_list_store_append (model, &iter);
 		gtk_list_store_set (model, &iter,
 				    ACTIVE_COLUMN, gedit_plugins_engine_plugin_is_active (info),
-				    NAME_COLUMN, info,
+				    AVAILABLE_COLUMN, gedit_plugins_engine_plugin_is_available (info),
+				    INFO_COLUMN, info,
 				    -1);
 
 		plugins = plugins->next;
@@ -298,25 +314,26 @@ plugin_manager_populate_lists (GeditPluginManager *pm)
 		gtk_tree_selection_select_iter (selection, &iter);
 
 		gtk_tree_model_get (GTK_TREE_MODEL (model), &iter,
-				    NAME_COLUMN, &info, -1);
+				    INFO_COLUMN, &info, -1);
 
 		gtk_widget_set_sensitive (GTK_WIDGET (pm->priv->configure_button),
 					  gedit_plugins_engine_plugin_is_configurable (info));
 	}
 }
 
-static void
+static gboolean
 plugin_manager_set_active (GtkTreeIter  *iter,
 			   GtkTreeModel *model,
 			   gboolean      active)
 {
 	GeditPluginInfo *info;
+	gboolean res = TRUE;
 	
 	gedit_debug (DEBUG_PLUGINS);
 
-	gtk_tree_model_get (model, iter, NAME_COLUMN, &info, -1);
+	gtk_tree_model_get (model, iter, INFO_COLUMN, &info, -1);
 
-	g_return_if_fail (info != NULL);
+	g_return_val_if_fail (info != NULL, FALSE);
 
 	if (active)
 	{
@@ -324,7 +341,8 @@ plugin_manager_set_active (GtkTreeIter  *iter,
 		if (!gedit_plugins_engine_activate_plugin (info)) {
 			gedit_debug_message (DEBUG_PLUGINS, "Could not activate %s.\n", 
 					     gedit_plugins_engine_get_plugin_name (info));
-			active ^= 1;
+					     
+			res = FALSE;
 		}
 	}
 	else
@@ -334,7 +352,7 @@ plugin_manager_set_active (GtkTreeIter  *iter,
 			gedit_debug_message (DEBUG_PLUGINS, "Could not deactivate %s.\n", 
 					     gedit_plugins_engine_get_plugin_name (info));
 
-			active ^= 1;
+			res = FALSE;
 		}
 	}
   
@@ -342,8 +360,12 @@ plugin_manager_set_active (GtkTreeIter  *iter,
 	gtk_list_store_set (GTK_LIST_STORE (model), 
 			    iter, 
 			    ACTIVE_COLUMN,
-			    gedit_plugins_engine_plugin_is_active (info), 
+			    gedit_plugins_engine_plugin_is_active (info),
+			    AVAILABLE_COLUMN,
+			    gedit_plugins_engine_plugin_is_available (info),			    
 			    -1);
+
+	return res;
 }
 
 static void
@@ -379,23 +401,21 @@ plugin_manager_get_selected_plugin (GeditPluginManager *pm)
 
 	if (gtk_tree_selection_get_selected (selection, NULL, &iter))
 	{
-		gtk_tree_model_get (model, &iter, NAME_COLUMN, &info, -1);
+		gtk_tree_model_get (model, &iter, INFO_COLUMN, &info, -1);
 	}
 	
 	return info;
 }
 
 static void
-plugin_manager_toggle_all (GeditPluginManager *pm)
+plugin_manager_set_active_all (GeditPluginManager *pm,
+			       gboolean            active)
 {
 	GtkTreeModel *model;
 	GtkTreeIter iter;
-	static gboolean active;
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	active ^= 1;
-
 	model = gtk_tree_view_get_model (GTK_TREE_VIEW (pm->priv->tree));
 
 	g_return_if_fail (model != NULL);
@@ -424,7 +444,7 @@ name_search_cb (GtkTreeModel *model,
 	gint key_len;
 	gboolean retval;
 
-	gtk_tree_model_get (model, iter, NAME_COLUMN, &info, -1);
+	gtk_tree_model_get (model, iter, INFO_COLUMN, &info, -1);
 	if (!info)
 		return FALSE;
 
@@ -448,6 +468,188 @@ name_search_cb (GtkTreeModel *model,
 	return retval;
 }
 
+static void
+enable_plugin_menu_cb (GtkMenu            *menu,
+		       GeditPluginManager *pm)
+{
+	GtkTreeModel *model;
+	GtkTreeIter iter;
+	GtkTreeSelection *selection;
+
+	model = gtk_tree_view_get_model (GTK_TREE_VIEW (pm->priv->tree));
+	g_return_if_fail (model != NULL);
+
+	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (pm->priv->tree));
+	g_return_if_fail (selection != NULL);
+
+	if (gtk_tree_selection_get_selected (selection, NULL, &iter))
+		plugin_manager_toggle_active (&iter, model);
+}
+
+static void
+enable_all_menu_cb (GtkMenu            *menu,
+		    GeditPluginManager *pm)
+{
+	plugin_manager_set_active_all (pm, TRUE);
+}
+
+static void
+disable_all_menu_cb (GtkMenu            *menu,
+		     GeditPluginManager *pm)
+{
+	plugin_manager_set_active_all (pm, FALSE);
+}
+
+static GtkWidget *
+create_tree_popup_menu (GeditPluginManager *pm)
+{
+	GtkWidget *menu;
+	GtkWidget *item;
+	GtkWidget *image;
+	GeditPluginInfo *info;
+
+	info = plugin_manager_get_selected_plugin (pm);
+
+	menu = gtk_menu_new ();
+
+	item = gtk_image_menu_item_new_with_mnemonic (_("_About"));
+	image = gtk_image_new_from_stock (GTK_STOCK_ABOUT,
+					  GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (about_button_cb), pm);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	item = gtk_image_menu_item_new_with_mnemonic (_("C_onfigure"));
+	image = gtk_image_new_from_stock (GTK_STOCK_PREFERENCES,
+					  GTK_ICON_SIZE_MENU);
+	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (configure_button_cb), pm);
+	gtk_widget_set_sensitive (item,
+				  gedit_plugins_engine_plugin_is_configurable (info));
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	item = gtk_check_menu_item_new_with_mnemonic (_("A_ctivate"));
+	gtk_widget_set_sensitive (item,
+				  gedit_plugins_engine_plugin_is_available (info));	
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
+					gedit_plugins_engine_plugin_is_active (info));
+	g_signal_connect (item, "toggled",
+			  G_CALLBACK (enable_plugin_menu_cb), pm);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);					
+
+	item = gtk_separator_menu_item_new ();
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	item = gtk_menu_item_new_with_mnemonic (_("Ac_tivate All"));
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (enable_all_menu_cb), pm);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+
+	item = gtk_menu_item_new_with_mnemonic (_("_Deactivate All"));
+	g_signal_connect (item, "activate",
+			  G_CALLBACK (disable_all_menu_cb), pm);
+	gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
+	
+	gtk_widget_show_all (menu);
+	
+	return menu;
+}
+
+static void
+tree_popup_menu_detach (GeditPluginManager *pm,
+			GtkMenu            *menu)
+{
+	pm->priv->popup_menu = NULL;
+}
+
+static void
+show_tree_popup_menu (GtkTreeView        *tree,
+		      GeditPluginManager *pm,
+		      GdkEventButton     *event)
+{
+	if (pm->priv->popup_menu)
+		gtk_widget_destroy (pm->priv->popup_menu);
+
+	pm->priv->popup_menu = create_tree_popup_menu (pm);
+	
+	gtk_menu_attach_to_widget (GTK_MENU (pm->priv->popup_menu),
+				   GTK_WIDGET (pm),
+				   (GtkMenuDetachFunc) tree_popup_menu_detach);
+
+	if (event != NULL)
+	{
+		gtk_menu_popup (GTK_MENU (pm->priv->popup_menu), NULL, NULL,
+				NULL, NULL,
+				event->button, event->time);
+	}
+	else
+	{
+		gtk_menu_popup (GTK_MENU (pm->priv->popup_menu), NULL, NULL,
+				gedit_utils_menu_position_under_tree_view, tree,
+				0, gtk_get_current_event_time ());
+
+		gtk_menu_shell_select_first (GTK_MENU_SHELL (pm->priv->popup_menu),
+					     FALSE);
+	}
+}
+
+static gboolean
+button_press_event_cb (GtkWidget          *tree,
+		       GdkEventButton     *event,
+		       GeditPluginManager *pm)
+{
+	/* We want the treeview selection to be updated before showing the menu.
+	 * This code is evil, thanks to Federico Mena Quintero's black magic.
+	 * See: http://mail.gnome.org/archives/gtk-devel-list/2006-February/msg00168.html
+	 * FIXME: Let's remove it asap.
+	 */
+
+	static gboolean in_press = FALSE;
+	gboolean handled;
+
+	if (in_press)
+		return FALSE; /* we re-entered */
+
+	if (GDK_BUTTON_PRESS != event->type || 3 != event->button)
+		return FALSE; /* let the normal handler run */
+
+	in_press = TRUE;
+	handled = gtk_widget_event (tree, (GdkEvent *) event);
+	in_press = FALSE;
+
+	if (!handled)
+		return FALSE;
+		
+	/* The selection is fully updated by now */
+	show_tree_popup_menu (GTK_TREE_VIEW (tree), pm, event);
+	return TRUE;
+}
+
+static gboolean
+popup_menu_cb (GtkTreeView        *tree,
+	       GeditPluginManager *pm)
+{
+	show_tree_popup_menu (tree, pm, NULL);
+	return TRUE;
+}
+
+static gint 
+model_name_sort_func (GtkTreeModel *model,
+		      GtkTreeIter  *iter1,
+		      GtkTreeIter  *iter2,
+		      gpointer      user_data)
+{
+	GeditPluginInfo *info1, *info2;
+	
+	gtk_tree_model_get (model, iter1, INFO_COLUMN, &info1, -1);
+	gtk_tree_model_get (model, iter2, INFO_COLUMN, &info2, -1);
+
+	return g_utf8_collate (gedit_plugins_engine_get_plugin_name (info1),
+			       gedit_plugins_engine_get_plugin_name (info2));
+}
+
 static void
 plugin_manager_construct_tree (GeditPluginManager *pm)
 {
@@ -457,40 +659,68 @@ plugin_manager_construct_tree (GeditPluginManager *pm)
 
 	gedit_debug (DEBUG_PLUGINS);
 
-	model = gtk_list_store_new (N_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_POINTER);
+	model = gtk_list_store_new (N_COLUMNS, G_TYPE_BOOLEAN, G_TYPE_BOOLEAN, G_TYPE_POINTER);
 
-	gtk_tree_view_set_model (GTK_TREE_VIEW (pm->priv->tree), GTK_TREE_MODEL (model));
+	gtk_tree_view_set_model (GTK_TREE_VIEW (pm->priv->tree),
+				 GTK_TREE_MODEL (model));
 	g_object_unref (model);
 
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (pm->priv->tree), TRUE);
-	gtk_tree_view_set_headers_clickable (GTK_TREE_VIEW (pm->priv->tree), TRUE);
+	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (pm->priv->tree), FALSE);
 
 	/* first column */
 	cell = gtk_cell_renderer_toggle_new ();
+	g_object_set (cell, "xpad", 6, NULL);
 	g_signal_connect (cell,
 			  "toggled",
 			  G_CALLBACK (active_toggled_cb),
 			  pm);
 	column = gtk_tree_view_column_new_with_attributes (PLUGIN_MANAGER_ACTIVE_TITLE,
 							   cell,
-							  "active",
+							   "active",
 							   ACTIVE_COLUMN,
+							   "activatable",
+							   AVAILABLE_COLUMN,
+							   "sensitive",
+							   AVAILABLE_COLUMN,
 							   NULL);
-	gtk_tree_view_column_set_clickable (column, TRUE);
-	gtk_tree_view_column_set_resizable (column, TRUE);
-	g_signal_connect (column, "clicked", G_CALLBACK (column_clicked_cb), pm);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (pm->priv->tree), column);
 
 	/* second column */
-	cell = gtk_cell_renderer_text_new ();
-	column = gtk_tree_view_column_new_with_attributes (PLUGIN_MANAGER_NAME_TITLE, cell, NULL);
+	column = gtk_tree_view_column_new ();
+	gtk_tree_view_column_set_title (column, PLUGIN_MANAGER_NAME_TITLE);
 	gtk_tree_view_column_set_resizable (column, TRUE);
-	gtk_tree_view_column_set_cell_data_func (column, cell, plugin_manager_view_cell_cb,
+
+	cell = gtk_cell_renderer_pixbuf_new ();
+	gtk_tree_view_column_pack_start (column, cell, FALSE);
+	g_object_set (cell, "stock-size", GTK_ICON_SIZE_SMALL_TOOLBAR, NULL);
+	gtk_tree_view_column_set_cell_data_func (column, cell,
+						 plugin_manager_view_icon_cell_cb,
 						 pm, NULL);
+	
+	cell = gtk_cell_renderer_text_new ();
+	gtk_tree_view_column_pack_start (column, cell, TRUE);
+	g_object_set (cell, "ellipsize", PANGO_ELLIPSIZE_END, NULL);
+	gtk_tree_view_column_set_cell_data_func (column, cell,
+						 plugin_manager_view_info_cell_cb,
+						 pm, NULL);
+	
+	
+	gtk_tree_view_column_set_spacing (column, 6);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (pm->priv->tree), column);
 
+	/* Sort on the plugin names */
+	gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (model),
+	                                         model_name_sort_func,
+        	                                 NULL,
+                	                         NULL);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model),
+					      GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID,
+					      GTK_SORT_ASCENDING);
+
 	/* Enable search for our non-string column */
-	gtk_tree_view_set_search_column (GTK_TREE_VIEW (pm->priv->tree), NAME_COLUMN);
+	gtk_tree_view_set_search_column (GTK_TREE_VIEW (pm->priv->tree),
+					 INFO_COLUMN);
 	gtk_tree_view_set_search_equal_func (GTK_TREE_VIEW (pm->priv->tree),
 					     name_search_cb,
 					     NULL,
@@ -505,14 +735,25 @@ plugin_manager_construct_tree (GeditPluginManager *pm)
 			  G_CALLBACK (row_activated_cb),
 			  pm);
 
+	g_signal_connect (pm->priv->tree,
+			  "button-press-event",
+			  G_CALLBACK (button_press_event_cb),
+			  pm);
+	g_signal_connect (pm->priv->tree,
+			  "popup-menu",
+			  G_CALLBACK (popup_menu_cb),
+			  pm);
 	gtk_widget_show (pm->priv->tree);
 }
 
 static void 
 gedit_plugin_manager_init (GeditPluginManager *pm)
 {
+	GtkWidget *label;
+	GtkWidget *alignment;
 	GtkWidget *viewport;
 	GtkWidget *hbuttonbox;
+	gchar *markup;
 
 	gedit_debug (DEBUG_PLUGINS);
 
@@ -520,6 +761,20 @@ gedit_plugin_manager_init (GeditPluginManager *pm)
 
 	gtk_box_set_spacing (GTK_BOX (pm), 6);
 
+	label = gtk_label_new (NULL);
+	markup = g_markup_printf_escaped ("<span weight=\"bold\">%s</span>",
+					  _("Active plugins"));
+	gtk_label_set_markup (GTK_LABEL (label), markup);
+	g_free (markup);
+	gtk_label_set_justify (GTK_LABEL (label), GTK_JUSTIFY_LEFT);
+	gtk_misc_set_alignment (GTK_MISC (label), 0.0, 0.5);
+	
+	gtk_box_pack_start (GTK_BOX (pm), label, FALSE, TRUE, 0);
+	
+	alignment = gtk_alignment_new (0., 0., 1., 1.);
+	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 12, 0);
+	gtk_box_pack_start (GTK_BOX (pm), alignment, TRUE, TRUE, 0);
+	
 	viewport = gtk_scrolled_window_new (NULL, NULL);
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (viewport),
 					GTK_POLICY_AUTOMATIC,
@@ -527,7 +782,7 @@ gedit_plugin_manager_init (GeditPluginManager *pm)
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (viewport), 
 					     GTK_SHADOW_IN);
 
-	gtk_box_pack_start (GTK_BOX (pm), viewport, TRUE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (alignment), viewport);
 
 	pm->priv->tree = gtk_tree_view_new ();
 	gtk_container_add (GTK_CONTAINER (viewport), pm->priv->tree);
@@ -573,6 +828,18 @@ gedit_plugin_manager_init (GeditPluginManager *pm)
 	}
 }
 
+static void
+gedit_plugin_manager_finalize (GObject *object)
+{
+	GeditPluginManager *pm = GEDIT_PLUGIN_MANAGER (object);
+
+	if (pm->priv->popup_menu)
+		gtk_widget_destroy (pm->priv->popup_menu);
+
+	G_OBJECT_CLASS (gedit_plugin_manager_parent_class)->finalize (object);
+
+}
+
 GtkWidget *gedit_plugin_manager_new (void)
 {
 	return g_object_new (GEDIT_TYPE_PLUGIN_MANAGER,0);
