@@ -41,13 +41,14 @@
 #include "gedit-prefs-manager-private.h"
 #include "gedit-debug.h"
 #include "gedit-encodings.h"
+#include "gedit-utils.h"
 
 #define DEFINE_BOOL_PREF(name, key, def) gboolean 			\
 gedit_prefs_manager_get_ ## name (void)					\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	return gedit_prefs_manager_get_bool (key,	\
+	return gedit_prefs_manager_get_bool (key,			\
 					     (def));			\
 }									\
 									\
@@ -56,7 +57,7 @@ gedit_prefs_manager_set_ ## name (gboolean v)				\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	gedit_prefs_manager_set_bool (key,		\
+	gedit_prefs_manager_set_bool (key,				\
 				      v);				\
 }									\
 				      					\
@@ -65,7 +66,7 @@ gedit_prefs_manager_ ## name ## _can_set (void)				\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	return gedit_prefs_manager_key_is_writable (key);\
+	return gedit_prefs_manager_key_is_writable (key);		\
 }	
 
 
@@ -75,7 +76,7 @@ gedit_prefs_manager_get_ ## name (void)			 		\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	return gedit_prefs_manager_get_int (key,		\
+	return gedit_prefs_manager_get_int (key,			\
 					    (def));			\
 }									\
 									\
@@ -84,7 +85,7 @@ gedit_prefs_manager_set_ ## name (gint v)				\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	gedit_prefs_manager_set_int (key,		\
+	gedit_prefs_manager_set_int (key,				\
 				     v);				\
 }									\
 				      					\
@@ -93,7 +94,7 @@ gedit_prefs_manager_ ## name ## _can_set (void)				\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	return gedit_prefs_manager_key_is_writable (key);\
+	return gedit_prefs_manager_key_is_writable (key);		\
 }		
 
 
@@ -103,7 +104,7 @@ gedit_prefs_manager_get_ ## name (void)			 		\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	return gedit_prefs_manager_get_string (key,	\
+	return gedit_prefs_manager_get_string (key,			\
 					       def);			\
 }									\
 									\
@@ -112,7 +113,7 @@ gedit_prefs_manager_set_ ## name (const gchar* v)			\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	gedit_prefs_manager_set_string (key,		\
+	gedit_prefs_manager_set_string (key,				\
 				        v);				\
 }									\
 				      					\
@@ -121,7 +122,7 @@ gedit_prefs_manager_ ## name ## _can_set (void)				\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	return gedit_prefs_manager_key_is_writable (key);\
+	return gedit_prefs_manager_key_is_writable (key);		\
 }		
 
 
@@ -154,11 +155,6 @@ static gint		 gedit_prefs_manager_get_int		(const gchar* key,
 static gchar		*gedit_prefs_manager_get_string		(const gchar* key, 
 								 const gchar* def);
 
-gboolean		 gconf_client_set_color 		(GConfClient* client, 
-								 const gchar* key,
-								 GdkColor val, 
-								 GError** err);
-
 
 gboolean
 gedit_prefs_manager_init (void)
@@ -298,17 +294,6 @@ gedit_prefs_manager_key_is_writable (const gchar* key)
 	return gconf_client_key_is_writable (gedit_prefs_manager->gconf_client, key, NULL);
 }
 
-static gchar* 
-gdk_color_to_string (GdkColor color)
-{
-	gedit_debug (DEBUG_PREFS);
-
-	return g_strdup_printf ("#%04x%04x%04x",
-				color.red, 
-				color.green,
-				color.blue);
-}
-
 static GdkColor
 gconf_client_get_color_with_default (GConfClient* client, const gchar* key,
                       		     const gchar* def, GError** err)
@@ -336,20 +321,22 @@ gconf_client_get_color_with_default (GConfClient* client, const gchar* key,
 	return color;
 }
 
-gboolean
-gconf_client_set_color (GConfClient* client, const gchar* key,
-                        GdkColor val, GError** err)
+static gboolean
+gedit_gconf_client_set_color (GConfClient   *client,
+                              const gchar   *key,
+                              GdkColor       val,
+                              GError       **err)
 {
 	gchar *str_color = NULL;
 	gboolean res;
-	
+
 	gedit_debug (DEBUG_PREFS);
 
 	g_return_val_if_fail (client != NULL, FALSE);
 	g_return_val_if_fail (GCONF_IS_CLIENT (client), FALSE);  
 	g_return_val_if_fail (key != NULL, FALSE);
 
-	str_color = gdk_color_to_string (val);
+	str_color = gedit_gdk_color_to_string (val);
 	g_return_val_if_fail (str_color != NULL, FALSE);
 
 	res = gconf_client_set_string (client,
@@ -390,7 +377,7 @@ gedit_prefs_manager_set_color (const gchar* key, GdkColor value)
 	g_return_if_fail (gconf_client_key_is_writable (
 				gedit_prefs_manager->gconf_client, key, NULL));
 			
-	gconf_client_set_color (gedit_prefs_manager->gconf_client, key, value, NULL);
+	gedit_gconf_client_set_color (gedit_prefs_manager->gconf_client, key, value, NULL);
 }
 
 
@@ -402,13 +389,22 @@ DEFINE_BOOL_PREF (use_default_font,
 /* Editor font */
 DEFINE_STRING_PREF (editor_font,
 		    GPM_EDITOR_FONT,
-		    GPM_DEFAULT_EDITOR_FONT);
+		    GPM_DEFAULT_EDITOR_FONT)
 
+/* System font */
+gchar *
+gedit_prefs_manager_get_system_font (void)
+{
+	gedit_debug (DEBUG_PREFS);
+
+	return gedit_prefs_manager_get_string (GPM_SYSTEM_FONT,
+					       GPM_DEFAULT_SYSTEM_FONT);
+}
 
 /* Use default colors */
 DEFINE_BOOL_PREF (use_default_colors,
 		  GPM_USE_DEFAULT_COLORS,
-		  GPM_DEFAULT_USE_DEFAULT_COLORS);
+		  GPM_DEFAULT_USE_DEFAULT_COLORS)
 
 
 /* Background color */	
@@ -523,7 +519,7 @@ gedit_prefs_manager_selection_color_can_set (void)
 /* Create backup copy */
 DEFINE_BOOL_PREF (create_backup_copy,
 		  GPM_CREATE_BACKUP_COPY,
-		  GPM_DEFAULT_CREATE_BACKUP_COPY);
+		  GPM_DEFAULT_CREATE_BACKUP_COPY)
 
 /* Backup extension. This is configurable only using gconftool or gconf-editor */
 gchar *
@@ -538,12 +534,12 @@ gedit_prefs_manager_get_backup_extension (void)
 /* Auto save */
 DEFINE_BOOL_PREF (auto_save,
 		  GPM_AUTO_SAVE,
-		  GPM_DEFAULT_AUTO_SAVE);
+		  GPM_DEFAULT_AUTO_SAVE)
 
 /* Auto save interval */
 DEFINE_INT_PREF (auto_save_interval,
 		 GPM_AUTO_SAVE_INTERVAL,
-		 GPM_DEFAULT_AUTO_SAVE_INTERVAL);
+		 GPM_DEFAULT_AUTO_SAVE_INTERVAL)
 
 
 /* Undo actions limit: if < 1 then no limits */
@@ -627,7 +623,7 @@ gedit_prefs_manager_wrap_mode_can_set (void)
 /* Tabs size */
 DEFINE_INT_PREF (tabs_size, 
 		 GPM_TABS_SIZE, 
-		 GPM_DEFAULT_TABS_SIZE);
+		 GPM_DEFAULT_TABS_SIZE)
 
 /* Insert spaces */
 DEFINE_BOOL_PREF (insert_spaces, 
@@ -647,7 +643,7 @@ DEFINE_BOOL_PREF (display_line_numbers,
 /* Toolbar visibility */
 DEFINE_BOOL_PREF (toolbar_visible,
 		  GPM_TOOLBAR_VISIBLE,
-		  GPM_DEFAULT_TOOLBAR_VISIBLE);
+		  GPM_DEFAULT_TOOLBAR_VISIBLE)
 
 
 /* Toolbar suttons style */
@@ -680,7 +676,6 @@ gedit_prefs_manager_get_toolbar_buttons_style (void)
 	g_free (str);
 
 	return res;
-
 }
 
 void
@@ -1097,10 +1092,10 @@ gedit_prefs_manager_get_auto_detected_encodings (void)
 
 		      g_return_val_if_fail (charset != NULL, NULL);
 		      enc = gedit_encoding_get_from_charset (charset);
-		      
+
 		      if (enc != NULL)
 		      {
-			      if (!data_exists (res, (const gpointer)enc))
+			      if (!data_exists (res, (gpointer)enc))
 				      res = g_slist_prepend (res, (gpointer)enc);
 
 		      }
@@ -1147,13 +1142,13 @@ gedit_prefs_manager_get_shown_in_menu_encodings (void)
 
 		      if (strcmp (charset, "CURRENT") == 0)
 			      g_get_charset (&charset);
-      
+
 		      g_return_val_if_fail (charset != NULL, NULL);
 		      enc = gedit_encoding_get_from_charset (charset);
-		      
+
 		      if (enc != NULL)
 		      {
-			      if (!data_exists (res, (const gpointer)enc))
+			      if (!data_exists (res, (gpointer)enc))
 				      res = g_slist_prepend (res, (gpointer)enc);
 		      }
 
@@ -1291,6 +1286,27 @@ gedit_prefs_manager_get_restore_cursor_position (void)
 					     GPM_DEFAULT_RESTORE_CURSOR_POSITION);
 }
 
+/* Global Lockdown */
+GeditLockdownMask
+gedit_prefs_manager_get_lockdown (void)
+{
+	guint lockdown = 0;
+
+	if (gedit_prefs_manager_get_bool (GPM_LOCKDOWN_COMMAND_LINE, FALSE))
+		lockdown |= GEDIT_LOCKDOWN_COMMAND_LINE;
+
+	if (gedit_prefs_manager_get_bool (GPM_LOCKDOWN_PRINTING, FALSE))
+		lockdown |= GEDIT_LOCKDOWN_PRINTING;
+
+	if (gedit_prefs_manager_get_bool (GPM_LOCKDOWN_PRINT_SETUP, FALSE))
+		lockdown |= GEDIT_LOCKDOWN_PRINT_SETUP;
+
+	if (gedit_prefs_manager_get_bool (GPM_LOCKDOWN_SAVE_TO_DISK, FALSE))
+		lockdown |= GEDIT_LOCKDOWN_SAVE_TO_DISK;
+
+	return lockdown;
+}
+
 /* The following functions are taken from gconf-client.c 
  * and partially modified. 
  * The licensing terms on these is: 
